C Program synth-mom-tensor.f
C Total field for a source with symmetric moment tensor M*D(t)
C Author: J. Pujol

C For the equations used see (9.13.11) and (10.10.9)-(10.10.11). 
C g, e, d, dd: Functions G, E, D, D dot 
C am: matrix of moment tensor
C an: near-field vector
C aia: intermediate-field vector for P vel.
C aib: intermediate-field vector for S vel.
C afp: far-field vector for P waves
C afs: far-field vector for S waves
C ga: vector gamma
C vth: vector theta
C vphi: vector phi
C amg: vector am*g
C trm: trace of M (scalar)
C gmg: ga*am*ga (scalar)
C tmg: ga*am*th (scalar)
C pmg: ga*am*phi (scalar)
C u(k,i): i-th component of the total displacement
C un(k,i): i-th component of the near field
C ui(k,i): i-th component of the intermediate field
C uf(k,i): i-th component of the far field

C NOTE 1: it was found that the time interval used in the synthetics depended
C on the source-observation point distance (r), with the smaller distances
C requiring smaller time intervals. The subroutine ddt has the required
C information, which was determined by trial and error.

C NOTE 2: The equations for the static displacement [see (10.10.4)-(10.10.6)] only
C apply to symmetric tensors with zero trace.

C NOTE 3: The program was used with an explosive source (moment tensor proportional 
C to the identity matrix) and it was found that the phi and theta components of the
C displacement are not identically equal to zero (see Problem 9.22), although they
C are several orders of magnitude smaller than the radial component. 

C Input file: synth-mt.dat

        parameter (nss=1000000,npp=1000000)
        dimension ti(npp),y(npp),ys(npp,3),yr(npp,3),ra(3,3),
     1tdec(50000),veloc(50000),accel(50000)
        double precision g(nss),e(nss),d(nss),dd(nss)
     1,am(3,3),an(3),aia(3),aib(3),afp(3),afs(3),ga(3),amg(3)
     1,u(npp,3),un(npp,3),ui(npp,3),uf(npp,3),cn(npp,3),vth(3),
     1vphi(3),ustatic(3),ytemp(3)
        double precision at,vp,vs,r,a,aa,ai,aainv,trm,gmg,tmg,pmg,
     1r4i,ar3i,a2r2i,a3ri,br3i,b2r2i,b3ri,dt,c1,c2,c3,rho,pi,rhopi,
     1th,phi,t,eat,rad,rkm
	double precision facpr,facsr,facpth,facsth,facpphi,facsphi
c	common /radpat/facpr,facsr,facpth,facsth,facpphi,facsphi,
c     1th,phi,rhopi,vp,vs,r,rad

	character cnum*2,cnum2*7,char*3,char2*20,card*60

	open(1,name='synth-mt.dat',status='old')
        call dashdc('$''''''$',2,5)
	call opngks        
	call setusv('LW',2000)

C INPUT PARAMETERS
	read(1,*)vp,vs,rho,th,phi
	vp=vp*1.d+5	!km/s; km converted to cm
	vs=vs*1.d+5	!km/s; km converted to cm
	pi=4*datan(1.d0)
	rad=pi/180

	th=th*rad
	phi=phi*rad
	rhopi=1./(4*pi*rho)

	read(1,*)a	!exponent
	ai=1./a
	aa=a*a
        aainv=1./aa

	call title(rad,a,th,phi)

	read(1,*)((am(ii,jj),jj=1,3),ii=1,3)	!moment tensor; row-wise
C Check that the tensor is symmetric
	do ii=1,3
	do jj=1,ii
	if(am(ii,jj).ne.am(jj,ii))then
	print*,ii,jj,am(ii,jj),am(jj,ii)
	print*,' THE MOMENT TENSOR IS NOT SYMMETRIC. STOP!!!'
	stop
	end if
	end do
	end do

	irot=1

1	read(1,*,end=1000,err=1000)rkm,wl,wr,wb1,wb2,iuva

	if(iuva.eq.1)then
	card(1:1)='u'
	elseif(iuva.eq.2)then
	card(1:1)='v'
	elseif(iuva.eq.3)then
	card(1:1)='a'
	end if

        call ddt(rkm,dt,np,ndec)
	ns=np
	r=rkm*1.d+5	!km converted to cm

        dtt=sngl(dt)
        do i=1,np
        ti(i)=(i-1)*dtt
	end do

        do i=1,ns
        t=(i-1)*dt
        at=a*t
        eat=dexp(-at)
C See (10.10.9)-(10.10.12) and (9.5.21)-(9.5.23)
        g(i)=.5*aainv*(6-6*eat+at*at-4*at-2*at*eat)	
        e(i)=t-2*ai*(1-eat)+t*eat
        d(i)=1-eat-at*eat
        dd(i)=aa*t*eat	!d dot
        end do

C See (9.9.16)
	ga(1)=dsin(th)*dcos(phi)
	ga(2)=dsin(th)*dsin(phi)
	ga(3)=dcos(th)
c	print*,' gamma:',ga
	vth(1)=dcos(th)*dcos(phi)
	vth(2)=dcos(th)*dsin(phi)
	vth(3)=-dsin(th)
	vphi(1)=-dsin(phi)
	vphi(2)=dcos(phi)
	vphi(3)=0

	trm=am(1,1)+am(2,2)+am(3,3)

		do i=1,3
		amg(i)=0
		do j=1,3
	amg(i)=amg(i)+am(i,j)*ga(j)
		end do
		end do
c	print*,' M * gamma:',amg
	
	gmg=0
	tmg=0
	pmg=0
		do i=1,3
	gmg=gmg+ga(i)*amg(i)
	tmg=tmg+vth(i)*amg(i)
	pmg=pmg+vphi(i)*amg(i)
		end do
c	print*,' gamma * M * gamma:',gmg

C See (9.13.3)-(9.13.8)
		do i=1,3
	an(i)=3*(5*gmg*ga(i)-trm*ga(i)-2*amg(i))*rhopi
	aia(i)=(6*gmg*ga(i)-trm*ga(i)-2*amg(i))*rhopi
	aib(i)=(6*gmg*ga(i)-trm*ga(i)-3*amg(i))*rhopi
	afp(i)=gmg*ga(i)*rhopi
	afs(i)=(gmg*ga(i)-amg(i))*rhopi
		end do

	r4i=1./(r**4)
	ar3i=1./(vp*r**3)
	a2r2i=1./((vp*r)**2)
	a3ri=1./(r*vp**3)
	br3i=1./(vs*r**3)
	b2r2i=1./((vs*r)**2)
	b3ri=1./(r*vs**3)
	nrvp=(r/vp)/dt
	nrvs=(r/vs)/dt
c	print*,nrvp,nrvs

        do k=1,np
        do i=1,3
        u(k,i)=0
        un(k,i)=0
        ui(k,i)=0
        uf(k,i)=0
        end do
        end do

		do i=1,3
	do ii=1,ns
	k=nrvp+ii-1
	if(k.gt.np)go to 10
	un(k,i)=an(i)*(r4i*g(ii)+ar3i*e(ii))
	ui(k,i)=aia(i)*a2r2i*d(ii)
	uf(k,i)=afp(i)*a3ri*dd(ii)
	u(k,i)=un(k,i)+ui(k,i)+uf(k,i)
	end do

10	do ii=1,ns
	k=nrvs+ii-1
	if(k.gt.np)go to 30
	c1=an(i)*(r4i*g(ii)+br3i*e(ii))
	cn(k,i)=c1
	c2=aib(i)*b2r2i*d(ii)
	c3=afs(i)*b3ri*dd(ii)
	un(k,i)=un(k,i)-c1
	ui(k,i)=ui(k,i)-c2
	uf(k,i)=uf(k,i)-c3
	u(k,i)=u(k,i)-(c1+c2+c3)
	end do
30		end do

        tp=sngl(r/vp)
        ts=sngl(r/vs)
	xl=10*aint(tp/10)
	xr=xl+45
	xlr=.5*(xl+xr)
	yb=-1.05
	yt=1.24
	
	isymbsiz=100
	call setusv('MS',isymbsiz)

	call pcseti('CD',1)    !duplex character set

		do i=1,3
	npdec=0
	do k=1,np,ndec
        npdec=npdec+1
	ys(npdec,i)=sngl(u(k,i))
        tdec(npdec)=ti(k)
        end do
c        print*,npdec
		end do

C Static displacement [see (10.10.4)-(10.10.6)]
	ustatic(1)=tmg*rhopi*(1./r**2)*(1./vp**2)
	ustatic(2)=pmg*rhopi*(1./r**2)*(1./vp**2)
	ustatic(3)=gmg*.5*rhopi*(1./r**2)*((3./vs**2)-(1./vp**2))
c	print*,ustatic

C Rotation matrix to go from Cartesian to spherical coordinates [see (10.10.7)]
	ths=sngl(th)
	phis=sngl(phi)
	ct=cos(ths)
	st=sin(ths)	
	cp=cos(phis)
	sp=sin(phis)
C Rotation matrix
	ra(1,1)=ct*cp
	ra(1,2)=ct*sp
	ra(1,3)=-st
	ra(2,1)=-sp
	ra(2,2)=cp
	ra(2,3)=0
	ra(3,1)=st*cp
	ra(3,2)=st*sp
	ra(3,3)=ct
c	print*,' ra:',ra

			if(irot.ne.1)then
C Rotate the static values with the TRANSPOSE matrix to go to Cartesian coord.
		do k=1,3
		ytemp(k)=0
	do l=1,3
	ytemp(k)=ytemp(k)+ra(l,k)*ustatic(l)
	end do
		end do
	do i=1,3
	ustatic(i)=ytemp(i)
	end do

		do i=1,3
		do j=1,3
		ra(i,j)=0
		end do
		end do
		do j=1,3
		ra(j,j)=1	!identity matrix
		end do
			end if

			do k=1,npdec
		do i=1,3
		yr(k,i)=0
	do l=1,3
	yr(k,i)=yr(k,i)+ra(i,l)*ys(k,l)
	end do
		end do
			end do

        call set(wl,wr,wb2-.05,wb1+.15,xl-5,xr,yb,yt,1)  
        call line(0,-.5,xr,-.5)
        tpdec=tdec(npdec)
	tpdec=xr
        deltat=10
        nn=tpdec/deltat
        do i=1,nn+1
        tt=(i-1)*deltat
        call line(tt,-.5,tt,-.55)
        itt=tt
        write(cnum,'(i2)')itt
        call plchhq(tt,-.65,cnum,10.,0,0)
        end do
        call plchhq(xlr,-.8,'Time (s)',10.,0,0)

			do l=1,3,2
	if(l.eq.1)then
	card(2:10)=':B::1010:'
	wb=wb1
	elseif(l.eq.3)then
	card(2:10)=':B::1003:'
	wb=wb2
	end if
	wt=wb+.15
	
	call set(wl,wr,wb,wt,xl-5,xr,yb,yt,1)  

	call plchhq(xl,1.05,card(1:10),13.,0,1.)

	do k=1,npdec
	y(k)=yr(k,l)
	end do

        if(iuva.eq.1)go to 50
        if(iuva.eq.3)go to 40
C Compute velocity
        do i=2,npdec-1
        veloc(i)=.5*(y(i+1)-y(i-1))
        end do
        veloc(1)=veloc(2)
        veloc(npdec)=veloc(npdec-1)

        do i=1,npdec
        y(i)=veloc(i)
        end do
        go to 50

C Compute acceleration
40        do i=2,npdec-1
        accel(i)=.5*(y(i+1)-y(i-1))
        end do
        accel(1)=accel(2)
        accel(npdec)=accel(npdec-1)

        do i=1,npdec
        y(i)=accel(i)
        end do

50	amax=0
	do i=1,npdec
	absy=abs(y(i))
	if(amax.lt.absy)amax=absy
	end do
c	write(6,*)amax
	write(cnum2,20)amax
20	format(1pe7.1)
        call plchhq(xr,1.15,cnum2,10.,0,1.)

        ir=rkm
	if(ir.lt.10)then
        write(char(1:1),'(i1)')ir        
	ic2=1
	elseif(ir.lt.100)then
        write(char(1:2),'(i2)')ir        
	ic2=2
	elseif(ir.lt.1000)then
        write(char(1:3),'(i3)')ir        
	ic2=3
	end if
        char2='r = '//char//' km'
        char2='r = '//char(1:ic2)//' km'
        CALL PLCHHQ(.5*(xl+xr),1.15,char2,10.,0.,0)


	fac=1./amax
	do k=1,npdec
	y(k)=fac*y(k)
	end do

        tp=sngl(r/vp)
        ts=sngl(r/vs)
        call line(tp,0,tp,.2)
        call line(ts,0,ts,-.2)
	call plchhq(tp,0.07,':1347:',8.,180.,0)
	call plchhq(ts,-0.07,':1347:',8.,0.,0)
        call lined(0,0,xr,0)

c	call setusv('LW',3000)
	call curve(tdec,y,npdec)
c	if(irot.ne.1)go to 90
	ust=sngl(ustatic(l))
        tpdec=tdec(npdec)
	ust=fac*ust
        if(iuva.eq.1)call line(tpdec,ust,xr,ust)

90			end do
					go to 1
1000	call frame
	call clsgks
	stop
	end 

        subroutine ddt(r,dt,np,ndec)
        double precision r,dt
        if(r.le.5.1)then
        dt=0.00002      !sec
        np=1000000
        ndec=5000       !n decimate
        return
        elseif(r.gt.5.1.and.r.le.15)then
        dt=0.00005
        np=400000
        ndec=2000
        elseif(r.gt.15.and.r.le.50)then
        dt=0.0001
        np=800000
        ndec=1000
        elseif(r.gt.50.and.r.le.1001)then
        dt=0.001
        np=100000
        ndec=100
        end if
        
        return
        end
C........................................................................
	subroutine title(rad,a,th,phi)
	double precision rad,a,th,phi
	character*100 card

	card='                                                  '
	card(1:4)=' a= '
	write(card(5:7),'(f3.1)')a
	card(14:14)='='
	write(card(15:19),'(f5.1)')th/rad
	card(25:25)='='
	write(card(26:30),'(f5.1)')phi/rad
	call plchhq(.5,.95,card(1:34),12.,0,0.)
	card='                                                  '
	card(10:15)=':1210:'
	card(21:26)=':1225:'
	call plchhq(.5,.95,card(1:34),25.,0,0.)
	card='                                                  '
	card(31:39)=':S::1331:'
	call plchhq(.5,.95,card(1:39),12.,0,0.)
	card='                                                  '
	card(7:15)=':S::1331:'
	call plchhq(.5,.95,card(1:15),12.,0,0.)

	return
	end

